// Transputer boot, read and write functions.

// usage:       #pragma hdrstop
//              #include "transput.c"

#include <dos.h>
#include <io.h>
#include <fcntl.h>


// START of constants to use transputers.


#define C012_BASE_ADDR      0x150
#define C012_IDR_OFFSET     0x00  /* input data register offset from base */
#define C012_ODR_OFFSET     0x01  /* output data register offset from base */
#define C012_ISR_OFFSET     0x02  /* input status register offset from base */
#define C012_OSR_OFFSET		0x03  /* output status register offset from base */
#define C012_RESET_OFFSET	0x10  /* address for resetting the link */
#define C012_ANALYSE_OFFSET 0x11  /* address for analysing the link */
#define C012_DMA_RQ			0x12  /* address of DMA read/write init */
#define C012_INT_EN_OFFSET 	0x13  /* interrupt enable address */
#define DMA_INT_SELECT		0x14  /* DMA & Interrupt channel select reg */
#define SM_SYSTEM_RESET     1
#define SET_ANALYSE_LOW     0
#define RESET_RESET         0
#define BUFFERSIZE          (1024*8)
#define ANALYSE_TIME_OUT    1000L
#define TRANSPUTER          0x150
#define EOI              	0x20  /* End of Interrupt RESET on 8259 contr. */

unsigned int    C012_idr    = C012_BASE_ADDR + C012_IDR_OFFSET,
				C012_odr    = C012_BASE_ADDR + C012_ODR_OFFSET,
				C012_isr    = C012_BASE_ADDR + C012_ISR_OFFSET,
				C012_osr    = C012_BASE_ADDR + C012_OSR_OFFSET,
				C012_reset  = C012_BASE_ADDR + C012_RESET_OFFSET,
				C012_analyse= C012_BASE_ADDR + C012_ANALYSE_OFFSET,
				C012_int_en = C012_BASE_ADDR + C012_INT_EN_OFFSET,
				C012_intsel = C012_BASE_ADDR + DMA_INT_SELECT,
				C012_dma_rq = C012_BASE_ADDR + C012_DMA_RQ;



#define BOOT_OK                 1
#define BOOT_NOFILE             2
#define BOOT_NOWRITELINK        3
// END of constants to use transputers.


/* ------------------ function prototypes ------------------------------- */
// START of function declarations to use transputers.
int             boot_transputers(char *) ;
void    reset_transputers(void) ;
int             WriteLink(char *, unsigned int) ;
unsigned char ReadLinkByte(void) ;
int             ReadLinkINT16(void) ;
// END of function declarations to use transputers.
/* ---------------------------------------------------------------------- */

// START of global variables to use transputers.
char buffer[BUFFERSIZE] ;

// END of global variables to use transputers.
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
// START of functions to use transputers.
// START of functions to use transputers.
// START of functions to use transputers.
void reset_transputers(void)
				  {
				  outportb(C012_reset, RESET_RESET);
				  delay(11) ;
				  outportb(C012_analyse, SET_ANALYSE_LOW);
				  delay(11) ;
				  outportb(C012_reset, SM_SYSTEM_RESET);
				  delay(11) ;
				  outportb(C012_reset, RESET_RESET);
				  delay(11) ;
				  }

int boot_transputers(char *bootfilename)

// Boots the transputer system with the given bootstrap file.
// Returns:  BOOT_OK                         if ok
//           BOOT_NOFILE                     if cannot open btl file
//           BOOT_NOWRITELINK                if cannot write to link

{
	int  inhandle, Length=0 ;
	int  count = 0 ;

	if( (inhandle = open(bootfilename, O_RDONLY | O_BINARY)) < 0)
	{
		 return BOOT_NOFILE ;
	}

	reset_transputers() ;

	while( (Length = read(inhandle, buffer, BUFFERSIZE)) > 0)
	{
		count = WriteLink(buffer, Length) ;
		if (count != Length)
		{
			close(inhandle) ;   /* close btl file */
			return BOOT_NOWRITELINK ;
		}
	}

	close(inhandle) ;   /* close btl file */
	return BOOT_OK ;
}

int WriteLink(char *buffer, unsigned int counter)
{
	register int  n;
	register long l;

#ifdef _KLP_DEBUG
	char s[200];
	sprintf(s, "\nWriteLink(%d, %d) start: press any key!!!\n", *buffer, counter) ;
		if (graph)
			outtextxy(0, 100, s);
		else
			printf(s) ;
		getch() ;
#endif /* _KLP_DEBUG */


	for(n=0; counter--; n++)
	{
		for(l=0 ;;)
			if (inportb(C012_osr) & 1)
			{
				outportb(C012_odr, (*buffer++ & 0xFF));
				break;
			}
			else
			{
				if (++l == 100000L)
				return(n);
			}
	 }

#ifdef _KLP_DEBUG
	 sprintf(s, "WriteLink(%d, %d) ended\n", *buffer, counter) ;
	 if (graph)
			outtextxy(0, 100, s);
	 else
			printf(s) ;
#endif /* _KLP_DEBUG */


	 return(n);
}

unsigned char ReadLinkByte()
{
	register int  n;

	int succ = FALSE;

	while(!succ)
		if(inportb(C012_isr) & 1)
		{
			n = inportb(C012_idr) & 0xFF;
			succ = TRUE;
		}

	return n ;
}

int ReadLinkINT16()
{
	register int  n1, n2;
	int succ = FALSE;

	while(!succ)
		if(inportb(C012_isr) & 1)
		{
			 n1 = inport(C012_idr) & 0xFF;
			 succ = TRUE;
		}

		succ = FALSE;

	while(!succ)
		 if(inportb(C012_isr) & 1)
		{
		 n2 = inport(C012_idr) & 0xFF;
		 succ = TRUE;
		}

		return (n2 << 8) + n1 ;
  }
// END of functions to use transputers.
// END of functions to use transputers.
// END of functions to use transputers.
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
//         if ( !boot_transputers("playfast.btl") )
//                WriteLink(&count, 1) ;
