// INT_COM1.C
// Programme to test three interrupts Link output, Link input & Error.
// Copyright IC-Routing Ltd.
// Author B C O'Neill
#include <stdio.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>

#define TRUE  1
#define FALSE 0
#define TIME_LIMIT 5  // 5 seconds time out

#include "transput.c"  //Transputer board address and functions

#define INTR_BASE1       0x08            // IRQ(0-7) offset vale of handler
#define INTR_BASE2       0x68            // IRQ(8-15) offset vale of handler

void interrupt (*oldhandler)(void);

char data;
int irq_no, irq_handle, oldstate;
int count = 0, cnt = 0;
char array[1000];
int finito = FALSE;

void interrupt handler_out(void)
{
	data = array[cnt];
	//printf("\nOutport status: %X", inportb(C012_osr) & 1);
	outportb(C012_odr, data);
	cnt++;
	if(cnt == (count-1))
	{
		finito = TRUE;
		outportb(C012_int_en, 0x00);     // Disable LinkOut Interrupt Enable
		outportb(C012_osr, 0x00);        // Disable LinkOut Interrupt on C012
	}
	outportb(0x20, EOI);    /* Resetting Interrupt Controller */
  outportb(0x0a0, EOI);    /* Resetting Interrupt Controller */
}

void interrupt handler_in(void)
{
	data = inportb(C012_idr);
	array[cnt] = data;
	cnt++;
	//printf("\nInport status: %X", inportb(C012_isr) & 1);
	if(cnt == (count - 1))
	{
		finito = TRUE;
		outportb(C012_int_en, 0x00);     // Disable LinkIn Interrupt Enable
		outportb(C012_isr, 0x00);        // Disable LinkIn Interrupt on C012
	}
	outportb(0x20, EOI);    /* Resetting Interrupt Controller */
 	outportb(0x0a0, EOI);    /* Resetting Interrupt Controller */
}

void interrupt handler_error(void)
{
	printf("\nTransputer error detected by interrupt.");
	finito = TRUE;
	printf("\nError flag: %d", inportb(C012_reset) & 1);
  outportb(0x20, EOI);    /* Resetting Interrupt Controller */
	outportb(0x0a0, EOI);    /* Resetting Interrupt Controller */
}

void init(void)
{
	int  i, state,  msk, sel;
	// char ch[1];
	printf("\n\nStarting IRQ test program:");
	printf("\n");
	printf("\nIRQ channel number? Type 11 or 5");
	printf("\nChoose no. to match hardware switches:  ");
	scanf("%d", &irq_no);
	irq_handle      = INTR_BASE1 + irq_no;
  if (irq_no == 11)
   	irq_handle      = INTR_BASE2 + irq_no;

	// Get old interrupt function entry address:
	oldhandler = getvect(irq_handle);

	// Install your own interrupt handler routine:
	setvect(irq_handle, handler_out);
	//printf(" done.");
  if (irq_no == 5)
  {
	oldstate = inportb(0x21);            // Read state of Int controller mask
	printf("\nInterrupt Mask at 0x21: %2X", oldstate);
	msk = 0xFF - (1U << irq_no);
	//printf("\n IRQ Mask value :  %2x", (oldstate & msk));
	outportb(0x21, (oldstate & msk));  // enable IRQx on the interrupt mask
	state = inportb(0x21);            // confirm enabling IRQx
	printf("\nNew interrupt mask at 0x21: %2X", state);
	sel = 0x08;  // if IRQ11 selected
	if (irq_no == 5)
		sel = 0x09;
	outportb(C012_intsel, (sel));      // Select IRQ3, disable DMA
	//printf("\nInterrupt select register reads: %2X", inportb(C012_intsel));
  }
  else
  {
	oldstate = inportb(0xa1);            // Read state of Int controller mask
	printf("\nInterrupt Mask at 0x0a1: %2X", oldstate);
  msk = 0xFF - (1U << (irq_no-8));
 	//msk = 0xFF - 0x08;  // Mask bit 11
	//printf("\n IRQ Mask value :  %2x", (oldstate & msk));
	outportb(0xa1, (oldstate & msk));  // enable IRQx on the interrupt mask
	state = inportb(0xa1);            // confirm enabling IRQx
	printf("\nNew interrupt mask at 0x0a1: %2X", state);
	//sel = 0x08;  // if IRQ11 selected
	//if (irq_no == 5)
	//	sel = 0x09;
	//outportb(C012_intsel, (sel));      // Select IRQ3, disable DMA
	//printf("\nInterrupt select register reads: %2X", inportb(C012_intsel));
  }
}

void main(int argc, char *argv[])
{
  long l;
	int i, data_read;
	int  out;
	char ch, cc;
  clock_t start_time;

	init();
	boot_transputers("grab_int.btl");
	//boot_transputers(argv[1]);
	//printf("\nError flag: %d", inportb(C012_reset) & 1);

	printf("\nEnter a count and the numbers: ");
	scanf("%d", &count);
	for(i = 0; i < count; i++)
	{
		scanf("%d", &cc);
		array[i] = (unsigned char) cc;
	}
	ch = (char) count;

	WriteLink(&ch, 1);     // send count to transputers
	start_time = clock();
  outportb(C012_int_en, 0x04);    // Enable LinkOut Interrupt Enable
	outportb(C012_osr, 0x02);       // Enable LinkOut Interrupt on C012

	while(!finito)
	{
		if(((clock() - start_time) / CLK_TCK) >= TIME_LIMIT)
		{
			printf("\nLink output interrupt failed.");
			WriteLink(array, count);
			break;
    }
	}

	printf("\nLink output interrupt finished.");
	printf("  Data sent to transputer\n\n ");
	finito = FALSE;
	cnt = 0;
	count = ReadLinkByte();
  //printf("\nRead count ");
	// Change INT handler:
	setvect(irq_handle, handler_in);
	outportb(C012_int_en, 0x08);    // Enable LinkIn Interrupt Enable
	outportb(C012_isr, 0x02);       // Enable LinkIn Interrupt on C012

  start_time = clock();
	while(!finito)
	{
		if(((clock() - start_time) / CLK_TCK) >= TIME_LIMIT)
		{
			printf("\nLink input interrupt failed.");
			for (i=0; i < count; i++)
			  array[i] = ReadLinkByte();
			break;
    }
	}

	printf("\nData from transputer = Data in + 7");
	printf("\nCount = %d \n ", count);
	for(i = 0; i < count; i++)
		printf("%d ", array[i]);

	printf("\nError flag: %d", inportb(C012_reset) & 1);
	printf("\n\nLink input interrupt finished");
	printf("  Data received from transputer.  \n");
	finito = FALSE;
	// Change INT handler:
	setvect(irq_handle, handler_error);
	outportb(C012_int_en, 0x02);      // Enable LinkError Interrupt Enable
	WriteLink(&ch, 1);                // dummy write to delay error
  start_time = clock();
	while(!finito)
	{
		if(((clock() - start_time) / CLK_TCK) >= TIME_LIMIT)
		{
	    printf("\nError interrupt test failed.");
      break;
    }
	}

	printf("\n\nEnd of interrupt test.");
	setvect(irq_handle, oldhandler);
	if(irq_no < 8)
    outportb(0x21, oldstate );  // restoring old mask
  else
    outportb(0xa1, oldstate );  // restoring old mask
	printf("\nRestoring old handler and interrupt mask.\nPress any key.");
	getch();
}







